#include <bits/stdc++.h>
using namespace std;

int MAXN = 40;
int MAXM = 40;

int L = 8*MAXN*MAXM, R = 8*MAXN*MAXM;
vector<vector<int>> graph (L);
vector<int> dist (L);
vector<int> matchL (L, -1), matchR (R, -1);

bool bfs() {
  queue<int> Q;
  for (int l = 0; l < L; ++l)
    if (matchL[l] == -1) {
      dist[l] = 0;
      Q.push(l);
    } else
      dist[l] = -1;

  bool ans = false;
  while (!Q.empty()) {
    int l = Q.front(); Q.pop();
    for (auto r : graph[l])
      if (matchR[r] == -1)
        ans = true;
      else if (dist[matchR[r]] == -1) {
        dist[matchR[r]] = dist[l] + 1;
        Q.push(matchR[r]);
      }
  }

  return ans;
}

bool dfs(int l) {
  if (l == -1)
    return true;

  for (auto r : graph[l])
    if (matchR[r] == -1 ||
        dist[matchR[r]] == dist[l] + 1)
      if (dfs(matchR[r])) {
        matchR[r] = l;
        matchL[l] = r;
        return true;
      }

  return false;
}

int hopcroft() {
  int ans = 0;
  while (bfs())
    for (int l = 0; l < L; ++l)
      if (matchL[l] == -1 && dfs(l))
        ans++;

  return ans;
}

typedef struct {
    char digit = 0;
    bool orange;
} square;

#define LEFT 0
#define RIGHT 1
#define UP 2
#define DOWN 3

int main() {
    cin.tie(0);
    ios_base::sync_with_stdio(0);

    int n, m;
    cin >> n >> m;

    vector<vector<square>> grid (n, vector<square>(m));
    vector<vector<int>> by_row (n);
    vector<vector<int>> by_column (m);

    int orange_digit_count = 0;
    int blue_digit_count = 0;

    for (int i = 0; i < n; i++) {
    for (int j = 0; j < m; j++) {
        char c1, c2;
        cin >> c1 >> c2;

        if (c1 == '.') { continue; }

        by_row[i].push_back(j);
        by_column[j].push_back(i);
        grid[i][j] = {
            .digit = char(c1 - '0'),
            .orange = bool(c2 - 'a')
        };
        if (grid[i][j].orange) {
            orange_digit_count += grid[i][j].digit;
        } else {
            blue_digit_count += grid[i][j].digit;
        }
    }
    }

    if (orange_digit_count != blue_digit_count) {
        cerr << "o=" << orange_digit_count << " b=" << blue_digit_count << "\n";
        cout << "NO\n";
        return 0;
    }

    vector<vector<char>> nodes (n, vector<char>(m));

    for (int i = 0; i < n; i++) {
    for (int k = 1; k < by_row[i].size(); k++) {
        int lj = by_row[i][k-1];
        int j = by_row[i][k];

        if (grid[i][lj].orange != grid[i][j].orange) {
            int u = 8*(MAXN*i+lj)+2*RIGHT;
            int v = 8*(MAXN*i+j)+2*LEFT;
            if (!grid[i][j].orange) { swap(u, v); }
            graph[u].push_back(v);
            graph[u].push_back(v+1);
            graph[u+1].push_back(v);
            graph[u+1].push_back(v+1);
            // cerr << u << " " << v << "\n";
            // cerr << u << " " << v+1 << "\n";
            // cerr << u+1 << " " << v << "\n";
            // cerr << u+1 << " " << v+1 << "\n";
            nodes[i][lj] |= 1<<RIGHT;
            nodes[i][j] |= 1<<LEFT;
        }
    }
    }

    for (int j = 0; j < m; j++) {
    for (int k = 1; k < by_column[j].size(); k++) {
        int li = by_column[j][k-1];
        int i = by_column[j][k];

        if (grid[li][j].orange != grid[i][j].orange) {
            int u = 8*(MAXN*li+j)+2*DOWN;
            int v = 8*(MAXN*i+j)+2*UP;
            if (!grid[i][j].orange) { swap(u, v); }
            graph[u].push_back(v);
            graph[u].push_back(v+1);
            graph[u+1].push_back(v);
            graph[u+1].push_back(v+1);
            // cerr << u << " " << v << "\n";
            // cerr << u << " " << v+1 << "\n";
            // cerr << u+1 << " " << v << "\n";
            // cerr << u+1 << " " << v+1 << "\n";
            nodes[li][j] |= 1<<DOWN;
            nodes[i][j] |= 1<<UP;
        }
    }
    }

    for (int i = 0; i < n; i++) {
    for (int j = 0; j < m; j++) {
        for (int k = 0; k < (2*__builtin_popcount(nodes[i][j]) - grid[i][j].digit); k++) {
            for (int d = 0; d < 4; d++) if (nodes[i][j] & (1<<d)) {
                int u = 8*(MAXN*i+j)+k;
                int v = 8*(MAXN*i+j)+2*d;
                if (!grid[i][j].orange) {
                    // cerr << "b inv: " << v << " " << u << "\n";
                    // cerr << "b inv: " << v+1 << " " << u << "\n";
                    graph[v].push_back(u);
                    graph[v+1].push_back(u);
                } else {
                    // cerr << "o inv: " << u << " " << v << "\n";
                    // cerr << "o inv: " << u << " " << v+1 << "\n";
                    graph[u].push_back(v);
                    graph[u].push_back(v+1);
                }
            }
        }
    }
    }

    hopcroft();

    vector<vector<vector<int>>> lines (n, vector<vector<int>>(m, vector<int>(4)));

    bool ok = true;

    for (int i = 0; i < n; i++) {
    for (int j = 0; j < m; j++) {
        if (!grid[i][j].digit) { continue; }
        int used = 0;
        for (int d = 0; d < 4; d++) if (nodes[i][j] & (1<<d)) {
            int u = 8*(MAXN*i+j)+2*d;
            int v = 8*(MAXN*i+j)+2*d+1;

            // cerr << "orange? " << grid[i][j].orange << "\n";

            int used_now = 0;

            if (!grid[i][j].orange) {
                // cerr << "Checking for L match in " << u << " = " << matchL[u] << "\n";
                // cerr << "Checking for L match in " << v << " = " << matchL[v] << "\n";

                used += int(matchL[u]/8 != u/8);
                used += int(matchL[v]/8 != v/8);
                used_now += int(matchL[u]/8 != u/8);
                used_now += int(matchL[v]/8 != v/8);
            } else {
                // cerr << "Checking for R match in " << u << " = " << matchR[u] << "\n";
                // cerr << "Checking for R match in " << v << " = " << matchR[v] << "\n";

                used += int(matchR[u]/8 != u/8);
                used += int(matchR[v]/8 != v/8);
                used_now += int(matchR[u]/8 != u/8);
                used_now += int(matchR[v]/8 != v/8);
            }

            lines[i][j][d] = used_now;
        }

        if (used != grid[i][j].digit) {
            ok = false;
            // cerr << "Wrong got " << used << " but expected " << int(grid[i][j].digit) << "\n";
        }
    }
    }

    cout << (ok ? "YES" : "NO") << "\n";

    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            if (grid[i][j].digit == 0) {
                cerr << "   ";
            } else {
                cerr << " ";
                if (lines[i][j][UP] == 1) {
                    cerr << "|";
                } else if (lines[i][j][UP] == 2) {
                    cerr << "#";
                } else {
                    cerr << " ";
                }
                cerr << " ";
            }
        }
        cerr << "\n";

        for (int j = 0; j < m; j++) {
            if (grid[i][j].digit == 0) {
                cerr << "   ";
            } else {
                if (lines[i][j][LEFT] == 1) {
                    cerr << "-";
                } else if (lines[i][j][LEFT] == 2) {
                    cerr << "#";
                } else {
                    cerr << " ";
                }
                cerr << int(grid[i][j].digit);
                if (lines[i][j][RIGHT] == 1) {
                    cerr << "-";
                } else if (lines[i][j][RIGHT] == 2) {
                    cerr << "#";
                } else {
                    cerr << " ";
                }
            }
        }
        cerr << "\n";

        for (int j = 0; j < m; j++) {
            if (grid[i][j].digit == 0) {
                cerr << "   ";
            } else {
                cerr << " ";
                if (lines[i][j][DOWN] == 1) {
                    cerr << "|";
                } else if (lines[i][j][DOWN] == 2) {
                    cerr << "#";
                } else {
                    cerr << " ";
                }
                cerr << " ";
            }
        }

        cerr << "\n";
    }
}
